@@ -168,7 +168,18 @@ class Link(Element):
 
     #: (:class:`str`) The relation type of the link.  It corresponds to
     #: ``rel`` attribute of :rfc:`4287#section-4.2.7.2` (section 4.2.7.2).
-    relation = Attribute('rel', default=lambda _: 'alternate')
+    #:
+    #: .. seealso::
+    #:
+    #:    `Existing rel values`__ --- Microformats Wiki
+    #:        This page contains tables of known HTML ``rel`` values from
+    #:        specifications, formats, proposals, brainstorms, and non-trivial
+    #:        POSH_ usage in the wild.  In addition, dropped and rejected
+    #:        values are listed at the end for comprehensiveness.
+    #:
+    #:    __ http://microformats.org/wiki/existing-rel-values
+    #:    .. _POSH: http://microformats.org/wiki/POSH
+    relation = Attribute('rel', required=True, default=lambda _: 'alternate')
 
     #: (:class:`str`) The optional hint for the MIME media type of the linked
     #: content.  It corresponds to ``type`` attribute of
@@ -303,6 +314,18 @@ class LinkList(collections.MutableSequence):
         else:
             return link
 
+    @property
+    def favicon(self):
+        """(:class:`Link`) Find the link to a favicon, also known as
+        a shortcut or bookmark icon, if it exists.
+
+        .. versionadded:: 0.3.0
+
+        """
+        for link in self:
+            if 'icon' in link.relation.split():
+                return link
+
 
 # FIXME: it probably would be common for all specialized element list types
 LinkList.list_type = type('LinkList.list_type', (list, LinkList), {})
